GRIT
====

Lightweight [Backbone](http://documentcloud.github.com/backbone/)-like models for NodeJS and browsers. For times when
the whole of Backbone seems too hefty.

Sample
------
In pure JS:

```javascript
var User = Grit.extend(function(username) {
  Grit.call(this);
  this.set('username', username);
});

User.validate('username', function(username) {
  return typeof username === 'string';
});

var user = new User('reissbaker');
user.on('change:username', function(newUsername) {
  alert('changing username from ' + user.get('username') + ' to ' + newUsername;
});
```

Or, in CoffeeScript:

    class User extends Grit
      @validate 'username', (username) ->
        typeof username == string
  
      constructor: (username) ->
        super()
        @set 'username', username

    user = new User('reissbaker')
    user.on 'change:username', (newUsername) ->
      alert "changing username from #{user.get 'username'} to #{newUsername}"

API
---

The API follows the NodeJS EventEmitter API, and the events emitted are
inspired by Backbone.

### Constructor
* `new Grit(properties)` creates a new Grit object, and will set the
properties according to the properties hash passed in. Please note
that if you mark properties as required with `Grit.require`, you _must_ 
either pass their initial values into the constructor or set initial
values for them with `Grit.defaults`.

### Instance Methods
* `grit.on(e, handler)`, or `grit.addListener(e, handler)` add an event
handler to the object. Grit objects emit `"change:e"` events, where `e`
is the name of the property being changed.
* `grit.removeListener(e, handler)` removes an event handler.
* `grit.get(property)` gets a named property.
* `grit.set(property, value[, silent])` sets a property. Passing a truthy
value as the `silent` parameter suppresses a change event; otherwise, a
`change:property` event will be emitted, where `property` is the value
passed in as the `property` parameter.
* `grit.sync(model, data, property)` is an empty method, meant to be
overridden. It gets called after any property is changed. `model` is
the Grit object being changed, `data` is a Javascript object with the
model's data inside, and `property` is the name of the property that
was changed.
* `grit.load()` is another empty method. Override it to define how
to load models from the server or DB.

### Class Methods
* `Grit.defaults` accepts a hash of default values to set for properties
on model construction. If you don't specify default values for required
properties, you _must_ pass them into the constructor.
* `Grit.required(property)` marks a property as required. All required
properties that do not have default values set MUST be passed at 
initialization into the Grit constructor. If a required property is 
missing, all calls to `set` will fail to validate.
* `Grit.unrequired(property)` marks a formerly-required property as
no longer required.
* `Grit.validate(property, validator)` adds a validation callback to
a property. Validation callbacks must be of the form `function(value)`,
and the `value` that gets passed in is the new value being set. If
all validators of a property return truthy values, the property will 
successfully be set; otherwise, it won't.
* `Grit.extend(constructorFunction)` makes it easy for normal JS classes
to inherit from Grit, or any Grit subclass. Just pass the constructor 
function to `extend`, and don't forget to call `Grit.call(this)` (or
`SubClass.call(this)`) on the first line of the constructor function.
