(function() {
	var Grit, EventEmitter;

	/*
	 * require in NodeJS's native EventEmitter class,
	 * or emulate it in the browser.
	 */

	if(typeof exports !== 'undefined') {
		EventEmitter = require('events').EventEmitter;
	} else {
		EventEmitter = function() {
			var handlers = {};

			this.on = this.addListener = function(e, callback) {
				if(!handlers.hasOwnProperty(e)) {
					handlers[e] = [];
				}
				handlers[e].push(callback);
			};

			this.removeListener = function(e, callback) {
				var handleList, index, length, curr;
				if(handlers.hasOwnProperty(e)) {
					handleList = handlers[e];
					for(index = 0, length = handleList.length; index < length; index++) {
						curr = handleList[index];
						if(curr === callback) {
							handleList.splice(index, 1);
							if(handleList.length === 0)
								delete handlers[e];
							return;
						}	
					}
				}
			};

			this.emit = function(e) {
				var handleList, index, length, args;
				if(handlers.hasOwnProperty(e)) {
					handleList = handlers[e];
					args = Array.prototype.slice(this, arguments).slice(1);
					for(index = 0, length = handleList.length; index < length; index++)
						handleList[index].call(this, args);
				}
			};
		};
	}

	/*
	 * The model class itself.
	 */

	Grit = function(properties) {
		EventEmitter.call(this);
		var data, prop, valid, changeData, 
				validateData, ensurePresenceOfData;

		data = {};

		ensurePresenceOfData = function(that) {
			var prop;
			if(!that.constructor.hasOwnProperty('reqs') || !that.constructor.reqs)
				return true;

			for(prop in that.constructor.reqs) {
				if(that.constructor.reqs.hasOwnProperty(prop) && !data.hasOwnProperty(prop))
					return false;
			}
			return true;
		};
		
		validateData = function(that, name, value) {
			var validators, valid, index, length;
			if(!that.constructor.hasOwnProperty('validations') || !that.constructor.validations)
				return true;
			if(!that.constructor.validations.hasOwnProperty(name) || !that.constructor.validations[name])
				return true;
			else {
				validators = that.constructor.validations[name];
				for(index = 0, length = validators.length; index < length; index++) {
					if(!validators[index](value))
						return false;
				}
			}
			return true;
		};

		changeData = function(that, name, value, silent) {
			if(typeof silent === 'undefined' || !silent)
				that.emit('change:' + name, that, value);
			data[name] = value;
			that.sync(that, data, name);
		};

		this.get = function(name) {
			if(data.hasOwnProperty(name))
				return data[name];
			return null;
		};
		this.set = function(name, value, silent) {
			var valid = validateData(this, name, value) && ensurePresenceOfData(this);

			if(valid) {
				changeData(this, name, value, silent);
				return value;
			}

			return null;
		};

		// if there aren't properties, set them to empty
		if(typeof properties === 'undefined')
			properties = {};

		if(this.constructor.hasOwnProperty('defaults')) {
			for(prop in this.constructor.defaults) {
				if(this.constructor.defaults.hasOwnProperty(prop) && !properties.hasOwnProperty(prop))
					properties[prop] = this.constructor.defaults[prop];
			}
		}

		// check validity
		valid = true;
		if(this.constructor.hasOwnProperty('reqs')) {
			for(prop in this.constructor.reqs) {
				if(!properties.hasOwnProperty(prop) || !validateData(this, prop, properties[prop])) {
					valid = false;
					break;
				}
			}
		}	

		// set the defaults for things not being overridden; otherwise, set the properties
		if(valid)
			for(prop in properties)
				if(properties.hasOwnProperty(prop))
					changeData(this, prop, properties[prop], false);
	};
	Grit.prototype = new EventEmitter();
	Grit.prototype.constructor = Grit;

	// sync is left empty -- redefine it to automate
	// changes being synced to the server or db.
	Grit.prototype.sync = function(model, data, property) {
	};

	// load is left empty -- redefine it to automate
	// loading things from the server or db. load
	// is never auto-called, unlike sync()
	Grit.prototype.load = function() {
	};

	Grit.extend = function(structFn) {
		structFn.prototype = new Grit();
		structFn.prototype.constructor = structFn;
		for(var prop in Grit) {
			if(Grit.hasOwnProperty(prop))
				structFn[prop] = Grit[prop];
		}
		return structFn;
	};

	Grit.defaults = function(properties) {
		if(!this.hasOwnProperty('defaults'))
			this.defaults = {};
		for(var prop in properties)
			if(properties.hasOwnProperty(prop))
				this.defaults[prop] = properties[prop];
	};

	Grit.required = function() {
		var args, index; 
		args = Array.prototype.slice.call(arguments, 0);

		if(!this.hasOwnProperty('reqs'))
			this.reqs = {};

		for(index = 0; index < args.length; index++)
			this.reqs[args[index]] = true;
	};
	Grit.unrequired = function() {
		var args, index;
		args = Array.prototype.slice.call(arguments, 0);

		if(!this.hasOwnProperty('reqs'))
			return;

		for(index = 0; index < args.length; index++) {
			if(this.reqs.hasOwnProperty(args[index]))
				delete this.reqs[args[index]];
		}
	};

	Grit.validate = function(name, validator) {
		if(!this.hasOwnProperty('validations'))
			this.validations = {};

		if(!this.validations.hasOwnProperty(name))
			this.validations[name] = [];
		this.validations[name].push(validator);
	};
	Grit.removeValidation = function(name, validator) {
		var validators, index, length;
		if(!this.hasOwnProperty('validations'))
			return;

		if(this.validations.hasOwnProperty(name)) {
			validators = this.validations[name];
			for(index = 0, length = validators.length; index < length; index++) {
				if(validators[index] === validator) {
					validators.splice(index, 1);
					break;
				}
			}

			if(validators.length === 0)
				delete this.validations[name];
		}
	};

	/* 
	 * Export
	 */

	if(typeof exports !== 'undefined') {
		exports = module.exports = Grit;
	} else {
		window.Grit = Grit;
	}
}());
